@@ -1,6 +1,6 @@
 /* Parse dates for touch and date.
 
-   Copyright (C) 1998, 2003, 2005, 2007-2010 Free Software Foundation, Inc.
+   Copyright (C) 1998, 2003, 2005, 2007-2020 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Yacc-based version written by Jim Kingdon and David MacKenzie.
    Rewritten by Jim Meyering.  */
@@ -25,11 +25,14 @@
 # include <time.h>
 
 /* POSIX Date Syntax flags.  */
-# define PDS_LEADING_YEAR 1
-# define PDS_TRAILING_YEAR 2
-# define PDS_CENTURY 4
-# define PDS_SECONDS 8
-# define PDS_PRE_2000 16
+# define PDS_TRAILING_YEAR 1
+# define PDS_CENTURY 2
+# define PDS_SECONDS 4
+# define PDS_PRE_2000 8
+
+/* For compatibility with older versions of this header, in which
+   PDS_LEADING_YEAR had its own bit.  */
+# define PDS_LEADING_YEAR 0
 
 bool posixtime (time_t *p, const char *s, unsigned int syntax_bits);
 
