@@ -1,5 +1,5 @@
 /* id -- print real and effective UIDs and GIDs
-   Copyright (C) 1989-2013 Free Software Foundation, Inc.
+   Copyright (C) 1989-2020 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Written by Arnold Robbins.
    Major rewrite by David MacKenzie, djm@gnu.ai.mit.edu. */
@@ -26,10 +26,13 @@
 #include <selinux/selinux.h>
 
 #include "system.h"
+#include "die.h"
 #include "error.h"
 #include "mgetgroups.h"
 #include "quote.h"
 #include "group-list.h"
+#include "smack.h"
+#include "userspec.h"
 
 /* The official name of this program (e.g., no 'g' prefix).  */
 #define PROGRAM_NAME "id"
@@ -38,12 +41,22 @@
   proper_name ("Arnold Robbins"), \
   proper_name ("David MacKenzie")
 
-/* If nonzero, output only the SELinux context. -Z */
-static int just_context = 0;
-
-static void print_user (uid_t uid);
-static void print_full_info (const char *username);
-
+/* If nonzero, output only the SELinux context.  */
+static bool just_context = 0;
+/* If true, delimit entries with NUL characters, not whitespace */
+static bool opt_zero = false;
+/* If true, output the list of all group IDs. -G */
+static bool just_group_list = false;
+/* If true, output only the group ID(s). -g */
+static bool just_group = false;
+/* If true, output real UID/GID instead of default effective UID/GID. -r */
+static bool use_real = false;
+/* If true, output only the user ID(s). -u */
+static bool just_user = false;
+/* True unless errors have been encountered.  */
+static bool ok = true;
+/* If true, we are using multiple users. Terminate -G with double NUL. */
+static bool multiple_users = false;
 /* If true, output user/group name instead of ID number. -n */
 static bool use_name = false;
 
@@ -51,12 +64,13 @@ static bool use_name = false;
 static uid_t ruid, euid;
 static gid_t rgid, egid;
 
-/* True unless errors have been encountered.  */
-static bool ok = true;
-
 /* The SELinux context.  Start with a known invalid value so print_full_info
    knows when 'context' has not been set to a meaningful value.  */
-static security_context_t context = NULL;
+static char *context = NULL;
+
+static void print_user (uid_t uid);
+static void print_full_info (const char *username);
+static void print_stuff (const char *pw_name);
 
 static struct option const longopts[] =
 {
@@ -66,6 +80,7 @@ static struct option const longopts[] =
   {"name", no_argument, NULL, 'n'},
   {"real", no_argument, NULL, 'r'},
   {"user", no_argument, NULL, 'u'},
+  {"zero", no_argument, NULL, 'z'},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
   {NULL, 0, NULL, 0}
@@ -78,18 +93,22 @@ usage (int status)
     emit_try_help ();
   else
     {
-      printf (_("Usage: %s [OPTION]... [USERNAME]\n"), program_name);
+      printf (_("Usage: %s [OPTION]... [USER]...\n"), program_name);
       fputs (_("\
-Print user and group information for the specified USERNAME,\n\
-or (when USERNAME omitted) for the current user.\n\
-\n\
-  -a              ignore, for compatibility with other versions\n\
-  -Z, --context   print only the security context of the current user\n\
-  -g, --group     print only the effective group ID\n\
-  -G, --groups    print all group IDs\n\
-  -n, --name      print a name instead of a number, for -ugG\n\
-  -r, --real      print the real ID instead of the effective ID, with -ugG\n\
-  -u, --user      print only the effective user ID\n\
+Print user and group information for each specified USER,\n\
+or (when USER omitted) for the current user.\n\
+\n"),
+             stdout);
+      fputs (_("\
+  -a             ignore, for compatibility with other versions\n\
+  -Z, --context  print only the security context of the process\n\
+  -g, --group    print only the effective group ID\n\
+  -G, --groups   print all group IDs\n\
+  -n, --name     print a name instead of a number, for -ugG\n\
+  -r, --real     print the real ID instead of the effective ID, with -ugG\n\
+  -u, --user     print only the effective user ID\n\
+  -z, --zero     delimit entries with NUL characters, not whitespace;\n\
+                   not permitted in default format\n\
 "), stdout);
       fputs (HELP_OPTION_DESCRIPTION, stdout);
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
@@ -97,7 +116,7 @@ or (when USERNAME omitted) for the current user.\n\
 \n\
 Without any OPTION, print some useful set of identified information.\n\
 "), stdout);
-      emit_ancillary_info ();
+      emit_ancillary_info (PROGRAM_NAME);
     }
   exit (status);
 }
@@ -107,15 +126,8 @@ main (int argc, char **argv)
 {
   int optc;
   int selinux_enabled = (is_selinux_enabled () > 0);
-
-  /* If true, output the list of all group IDs. -G */
-  bool just_group_list = false;
-  /* If true, output only the group ID(s). -g */
-  bool just_group = false;
-  /* If true, output real UID/GID instead of default effective UID/GID. -r */
-  bool use_real = false;
-  /* If true, output only the user ID(s). -u */
-  bool just_user = false;
+  bool smack_enabled = is_smack_enabled ();
+  char *pw_name = NULL;
 
   initialize_main (&argc, &argv);
   set_program_name (argv[0]);
@@ -125,7 +137,7 @@ main (int argc, char **argv)
 
   atexit (close_stdout);
 
-  while ((optc = getopt_long (argc, argv, "agnruGZ", longopts, NULL)) != -1)
+  while ((optc = getopt_long (argc, argv, "agnruzGZ", longopts, NULL)) != -1)
     {
       switch (optc)
         {
@@ -134,11 +146,18 @@ main (int argc, char **argv)
           break;
 
         case 'Z':
-          /* politely decline if we're not on a selinux-enabled kernel. */
+          /* politely decline if we're not on a SELinux/SMACK-enabled kernel. */
+#ifdef HAVE_SMACK
+          if (!selinux_enabled && !smack_enabled)
+            die (EXIT_FAILURE, 0,
+                 _("--context (-Z) works only on "
+                   "an SELinux/SMACK-enabled kernel"));
+#else
           if (!selinux_enabled)
-            error (EXIT_FAILURE, 0,
-                   _("--context (-Z) works only on an SELinux-enabled kernel"));
-          just_context = 1;
+            die (EXIT_FAILURE, 0,
+                 _("--context (-Z) works only on an SELinux-enabled kernel"));
+#endif
+          just_context = true;
           break;
 
         case 'g':
@@ -153,6 +172,9 @@ main (int argc, char **argv)
         case 'u':
           just_user = true;
           break;
+        case 'z':
+          opt_zero = true;
+          break;
         case 'G':
           just_group_list = true;
           break;
@@ -164,48 +186,82 @@ main (int argc, char **argv)
     }
 
   size_t n_ids = argc - optind;
-  if (1 < n_ids)
-    {
-      error (0, 0, _("extra operand %s"), quote (argv[optind + 1]));
-      usage (EXIT_FAILURE);
-    }
 
   if (n_ids && just_context)
-    error (EXIT_FAILURE, 0,
-           _("cannot print security context when user specified"));
+    die (EXIT_FAILURE, 0,
+         _("cannot print security context when user specified"));
 
   if (just_user + just_group + just_group_list + just_context > 1)
-    error (EXIT_FAILURE, 0, _("cannot print \"only\" of more than one choice"));
+    die (EXIT_FAILURE, 0, _("cannot print \"only\" of more than one choice"));
 
-  bool default_format = (just_user + just_group + just_group_list
-                         + just_context == 0);
+  bool default_format = ! (just_user
+                           || just_group
+                           || just_group_list
+                           || just_context);
 
   if (default_format && (use_real || use_name))
-    error (EXIT_FAILURE, 0,
-           _("cannot print only names or real IDs in default format"));
+    die (EXIT_FAILURE, 0,
+         _("cannot print only names or real IDs in default format"));
 
-  /* If we are on a selinux-enabled kernel, no user is specified, and
+  if (default_format && opt_zero)
+    die (EXIT_FAILURE, 0,
+         _("option --zero not permitted in default format"));
+
+  /* If we are on a SELinux/SMACK-enabled kernel, no user is specified, and
      either --context is specified or none of (-u,-g,-G) is specified,
      and we're not in POSIXLY_CORRECT mode, get our context.  Otherwise,
      leave the context variable alone - it has been initialized to an
      invalid value that will be not displayed in print_full_info().  */
-  if (selinux_enabled
-      && n_ids == 0
+  if (n_ids == 0
       && (just_context
           || (default_format && ! getenv ("POSIXLY_CORRECT"))))
     {
       /* Report failure only if --context (-Z) was explicitly requested.  */
-      if (getcon (&context) && just_context)
-        error (EXIT_FAILURE, 0, _("can't get process context"));
+      if ((selinux_enabled && getcon (&context) && just_context)
+          || (smack_enabled
+              && smack_new_label_from_self (&context) < 0
+              && just_context))
+        die (EXIT_FAILURE, 0, _("can't get process context"));
     }
 
-  if (n_ids == 1)
+  if (n_ids >= 1)
     {
-      struct passwd *pwd = getpwnam (argv[optind]);
-      if (pwd == NULL)
-        error (EXIT_FAILURE, 0, _("%s: no such user"), argv[optind]);
-      ruid = euid = pwd->pw_uid;
-      rgid = egid = pwd->pw_gid;
+      multiple_users = n_ids > 1 ? true : false;
+      /* Changing the value of n_ids to the last index in the array where we
+         have the last possible user id. This helps us because we don't have
+         to declare a different variable to keep a track of where the
+         last username lies in argv[].  */
+      n_ids += optind;
+      /* For each username/userid to get its pw_name field */
+      for (; optind < n_ids; optind++)
+        {
+          struct passwd *pwd = NULL;
+          const char *spec = argv[optind];
+          /* Disallow an empty spec here as parse_user_spec() doesn't
+             give an error for that as it seems it's a valid way to
+             specify a noop or "reset special bits" depending on the system.  */
+          if (*spec)
+            {
+              if (parse_user_spec (spec, &euid, NULL, NULL, NULL) == NULL)
+                {
+                  /* parse_user_spec will only extract a numeric spec,
+                     so we lookup that here to verify and also retrieve
+                     the PW_NAME used subsequently in group lookup.  */
+                  pwd = getpwuid (euid);
+                }
+            }
+          if (pwd == NULL)
+            {
+              error (0, errno, _("%s: no such user"), quote (argv[optind]));
+              ok &= false;
+              continue;
+            }
+          pw_name = xstrdup (pwd->pw_name);
+          ruid = euid = pwd->pw_uid;
+          rgid = egid = pwd->pw_gid;
+          print_stuff (pw_name);
+          free (pw_name);
+        }
     }
   else
     {
@@ -221,7 +277,7 @@ main (int argc, char **argv)
           errno = 0;
           euid = geteuid ();
           if (euid == NO_UID && errno)
-            error (EXIT_FAILURE, errno, _("cannot get effective UID"));
+            die (EXIT_FAILURE, errno, _("cannot get effective UID"));
         }
 
       if (just_user ? use_real
@@ -230,7 +286,7 @@ main (int argc, char **argv)
           errno = 0;
           ruid = getuid ();
           if (ruid == NO_UID && errno)
-            error (EXIT_FAILURE, errno, _("cannot get real UID"));
+            die (EXIT_FAILURE, errno, _("cannot get real UID"));
         }
 
       if (!just_user && (just_group || just_group_list || !just_context))
@@ -238,40 +294,17 @@ main (int argc, char **argv)
           errno = 0;
           egid = getegid ();
           if (egid == NO_GID && errno)
-            error (EXIT_FAILURE, errno, _("cannot get effective GID"));
+            die (EXIT_FAILURE, errno, _("cannot get effective GID"));
 
           errno = 0;
           rgid = getgid ();
           if (rgid == NO_GID && errno)
-            error (EXIT_FAILURE, errno, _("cannot get real GID"));
+            die (EXIT_FAILURE, errno, _("cannot get real GID"));
         }
+        print_stuff (pw_name);
     }
 
-  if (just_user)
-    {
-      print_user (use_real ? ruid : euid);
-    }
-  else if (just_group)
-    {
-      if (!print_group (use_real ? rgid : egid, use_name))
-        ok = false;
-    }
-  else if (just_group_list)
-    {
-      if (!print_group_list (argv[optind], ruid, rgid, egid, use_name))
-        ok = false;
-    }
-  else if (just_context)
-    {
-      fputs (context, stdout);
-    }
-  else
-    {
-      print_full_info (argv[optind]);
-    }
-  putchar ('\n');
-
-  exit (ok ? EXIT_SUCCESS : EXIT_FAILURE);
+  return ok ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
 /* Convert a gid_t to string.  Do not use this function directly.
@@ -310,7 +343,7 @@ print_user (uid_t uid)
         {
           error (0, 0, _("cannot find name for user ID %s"),
                  uidtostr (uid));
-          ok = false;
+          ok &= false;
         }
     }
 
@@ -354,28 +387,28 @@ print_full_info (const char *username)
 
   {
     gid_t *groups;
-    int i;
 
-    int n_groups = xgetgroups (username, (pwd ? pwd->pw_gid : -1),
-                               &groups);
+    gid_t primary_group;
+    if (username)
+      primary_group = pwd ? pwd->pw_gid : -1;
+    else
+      primary_group = egid;
+
+    int n_groups = xgetgroups (username, primary_group, &groups);
     if (n_groups < 0)
       {
         if (username)
-          {
-            error (0, errno, _("failed to get groups for user %s"),
-                   quote (username));
-          }
+          error (0, errno, _("failed to get groups for user %s"),
+                 quote (username));
         else
-          {
-            error (0, errno, _("failed to get groups for the current process"));
-          }
-        ok = false;
+          error (0, errno, _("failed to get groups for the current process"));
+        ok &= false;
         return;
       }
 
     if (n_groups > 0)
       fputs (_(" groups="), stdout);
-    for (i = 0; i < n_groups; i++)
+    for (int i = 0; i < n_groups; i++)
       {
         if (i > 0)
           putchar (',');
@@ -392,3 +425,40 @@ print_full_info (const char *username)
   if (context)
     printf (_(" context=%s"), context);
 }
+
+/* Print information about the user based on the arguments passed. */
+
+static void
+print_stuff (const char *pw_name)
+{
+  if (just_user)
+      print_user (use_real ? ruid : euid);
+
+  /* print_group and print_group_lists functions return true on successful
+     execution but false if something goes wrong. We then AND this value with
+     the current value of 'ok' because we want to know if one of the previous
+     users faced a problem in these functions. This value of 'ok' is later used
+     to understand what status program should exit with. */
+  else if (just_group)
+    ok &= print_group (use_real ? rgid : egid, use_name);
+  else if (just_group_list)
+    ok &= print_group_list (pw_name, ruid, rgid, egid,
+                            use_name, opt_zero ? '\0' : ' ');
+  else if (just_context)
+    fputs (context, stdout);
+  else
+    print_full_info (pw_name);
+
+  /* When printing records for more than 1 user, at the end of groups
+     of each user terminate the record with two consequent NUL characters
+     to make parsing and distinguishing between two records possible. */
+  if (opt_zero && just_group_list && multiple_users)
+    {
+      putchar ('\0');
+      putchar ('\0');
+    }
+  else
+    {
+      putchar (opt_zero ? '\0' : '\n');
+    }
+}
