@@ -1,7 +1,7 @@
 #!/bin/sh
 # test for readlink mode.
 
-# Copyright (C) 2004-2014 Free Software Foundation, Inc.
+# Copyright (C) 2004-2020 Free Software Foundation, Inc.
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 # GNU General Public License for more details.
 
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 . "${srcdir=.}/tests/init.sh"; path_prepend_ ./src
 print_ver_ readlink
@@ -31,13 +31,13 @@ test "$v" = regfile || fail=1
 v=$(readlink link2) || fail=1
 test "$v" = missing || fail=1
 
-v=$(readlink subdir) && fail=1
+v=$(returns_ 1 readlink subdir) || fail=1
 test -z "$v" || fail=1
 
-v=$(readlink regfile) && fail=1
+v=$(returns_ 1 readlink regfile) || fail=1
 test -z "$v" || fail=1
 
-v=$(readlink missing) && fail=1
+v=$(returns_ 1 readlink missing) || fail=1
 test -z "$v" || fail=1
 
 Exit $fail
